import Component from './component';
import fetch from 'isomorphic-fetch';
import normalize from './normalize';
import _ from 'lodash';

class Client extends Component {

  /**
   * @ignore
   */
  defaults () {
    return {
      root: '/', // base url

      // throw Error on non-OK response?
      errors: true,

      fetch: {
        algo: fetch,
        options: {
          credentials: 'same-origin',
        },
      },
    };
  }

  constructor (options={}) {
    super(options);
    this.options = _.merge(this.defaults(), options);
  }

  /**
   * The fetch implementation
   * @type {Function}
   */
  get fetch () {
    if (!this._fetch) {
      this._fetch = this.option('fetch').algo;
    }
    return this._fetch;
  }

  /**
   * The fetch implementation
   * @type {Function}
   */
  set fetch (fetch) {
    this._fetch = fetch;
  }

  /**
   * The URL root
   * @readonly
   * @type {String}
   */
  get root () {
    return normalize.url(this.option('root'));
  }

  /**
   * Derive a fully qualified URL from the url root and the given path
   * @param {String} path
   * @return {String}
   */
  url (...tokens) {
    return `${this.root}${normalize.path(...tokens)}`;
  }

  /**
   * Create a fetch request options object
   * @param {Object} options
   * @return {Object}
   */
  fopts (options={}) {
    return {
      ...this.option('fetch').options,
      ...options,
    };
  }

  static createMethod (method) {
    Client.prototype[method] = function(path, fopts={}) {
      return this.fetch(this.url(path), this.fopts({
        ...fopts,
        method: method.toUpperCase(),
      }))
      .then((response) => {
        if (!response.ok && this.option('errors')) {
          let e = new Error('got non-OK response');
          e.response = response;
          throw e;
        }
        return response;
      })
    };
  }
}

['head', 'get', 'put', 'delete', 'post', 'patch'].forEach(Client.createMethod);

export default Client;

