const Client = lib;
const P = Promise;

describe('Client', () => {

  let client;
  let root = 'https://api.local';

  beforeEach(() => {
    client = new Client({
      root: root,
      errors: true,
      fetch: {
        algo: stub(),
        options: {},
      },
    });
  });

  ['head', 'get', 'put', 'delete', 'post', 'patch'].forEach((method) => {
    let http_method = method.toUpperCase();

    it(`sends a ${http_method} request`, () => {
      let {fetch} = client;
      let reqpath = '/foo/bar';
      let response = {ok: true};

      fetch.withArgs(`${root}${reqpath}`, match((opts) => {
        return opts.method === http_method;
      }))
      .returns(P.resolve(response));

      return client[method](reqpath)
      .then(() => expect(fetch.calledOnce).to.be.ok);
    })

    it('throws if the response is not ok', () => {
      let {fetch} = client;
      let reqpath = '/foo/bar';
      let response = {ok: false};
      let promise = P.resolve(response);

      fetch.returns(promise);

      return client[method](reqpath)
      .then(() => expect().fail())
      .catch((e) => {
        expect(e).to.be.an.instanceof(Error);
        expect(e.response).to.eql(response);
      });
    });

  });
});
